﻿    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;

    namespace CustomControls
    {
        [TemplatePart(Name="CLIPRECTANGLE", Type=typeof(RectangleGeometry))]
        public class ImageProgressBar : ProgressBar
        {
            public ImageProgressBar()
            {
                this.DefaultStyleKey = typeof(ImageProgressBar);
            }

            public ImageSource Source
            {
                get { return (ImageSource)GetValue(SourceProperty); }
                set { SetValue(SourceProperty, value); }
            }

            public static readonly DependencyProperty SourceProperty =
                DependencyProperty.Register("Source", typeof(ImageSource), typeof(ImageProgressBar), new PropertyMetadata(null));

            public Brush Fill
            {
                get { return (Brush)GetValue(FillProperty); }
                set { SetValue(FillProperty, value); }
            }

            public static readonly DependencyProperty FillProperty =
                DependencyProperty.Register("Fill", typeof(Brush), typeof(ImageProgressBar), new PropertyMetadata(null));

            private RectangleGeometry _clip;

            public override void OnApplyTemplate()
            {
                base.OnApplyTemplate();
                _clip = this.GetTemplateChild("CLIPRECTANGLE") as RectangleGeometry;
                this.ValueChanged += ImageProgressBar_ValueChanged;
                this.SizeChanged += ImageProgressBar_SizeChanged;
            }

            void ImageProgressBar_SizeChanged(object sender, SizeChangedEventArgs e)
            {
                UpdateClip();
            }

            void ImageProgressBar_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
            {
                UpdateClip();
            }

            private void UpdateClip()
            {
                if (_clip != null)
                {
                    _clip.Rect = new Rect(0, 0, this.ActualWidth, this.ActualHeight * ((this.Value - this.Minimum) / (this.Maximum - this.Minimum)));
                }
            }
        }
    }
